from ..args import PathArgument, convert_path_arguments


class SageMakerProcessingConfig(object):
    def __init__(self, dependencies=None, inputs=None, outputs=None):
        self.dependencies = dependencies or {}
        self.inputs =  convert_path_arguments(inputs)
        self.outputs =  convert_path_arguments(outputs)



import os
from pathlib import Path
from urllib.parse import urlparse
from urllib.request import pathname2url, url2pathname
from sagemaker.s3 import S3Uploader


def standardize_channel(channel):
    if os.path.exists(channel):
        return Path(os.path.abspath(channel)).as_uri()
    else:
        url = urlparse(channel)
        if url.scheme == 's3':
            return channel
        elif url.scheme in ['file', '']:
            path = url2pathname(url.path)
            if os.path.exists(path):
                return Path(path).as_uri()
            else:
                raise FileNotFoundError(
                    "Channel input [{}] does not exist".format(channel))


def standardize_channels(channels):
    return {k: standardize_channel(v) for k, v in channels.items()}


def upload_local_channel(channel, session, s3_uri):
    url = urlparse(channel)
    if url.scheme == 's3':
        return channel
    elif url.scheme == 'file':
        path = url2pathname(url.path)
        print("Uploading [{}] ([{}]) to [{}]".format(
            channel, path, s3_uri
        ))
        S3Uploader.upload(
            local_path=path,
            desired_s3_uri=s3_uri,
            sagemaker_session=session
        )
        return s3_uri
    else:
        raise ValueError("Unknown scheme: [{}]".format(url.scheme))


def upload_local_channels(channels, session, prefix):
    return {
        k: upload_local_channel(channel, session=session,
                                s3_uri="{}/{}".format(prefix, k))
        for k, channel in channels.items()
    }




from types import ModuleType
import os


def module_path(module):
    if isinstance(module, ModuleType):
        path = os.path.abspath(module.__file__)
        return os.path.dirname(path)
    else:
        return module




import subprocess
import os
import warnings
import re

def git_warn(name, e):
    warnings.warn("Cannot git_get_{}".format(name))
    warnings.warn(e)

def git_get_branch(file):
    cwd = os.path.dirname(os.path.abspath(file))
    try:
        proc = subprocess.run(
            ["git", "show-branch"], capture_output=True, cwd=cwd, check=True)
        branch = proc.stdout
        branch = branch.decode('utf-8')
        branch = branch.strip()
        if branch:
            return branch
        else:
            return None
    except Exception as e:
        git_warn("branch", e)
        return None


def git_get_status(file):
    cwd = os.path.dirname(os.path.abspath(file))
    try:
        proc = subprocess.run(
            ["git", "show", "--oneline", "-s"], capture_output=True, cwd=cwd, check=True)
        status = proc.stdout
        status = status.decode('utf-8')
        status = status.strip()
        if status:
            m = re.match("^(\\w+)\\s", status)
            if m:
                commit = m.group(1).strip()
            else:
                commit = None
            return status, commit
        else:
            return None, None
    except Exception as e:
        git_warn("status", e)
        return None, None


def git_get_remote(file, name="origin"):
    cwd = os.path.dirname(os.path.abspath(file))
    try:
        proc = subprocess.run(
            ["git", "remote", "get-url", name], capture_output=True, cwd=cwd, check=True)
        remote = proc.stdout
        remote = remote.decode('utf-8')
        remote = remote.strip()
        return remote
    except Exception as e:
        git_warn("remote", e)
        return None

def git_get_tags(__file__):
    status, commit = git_get_status(__file__)
    url = git_get_remote(__file__)
    branch = git_get_branch(__file__)
    tags = {
        "GitStatus": status,
        "GitCommit": commit,
        "GitOrigin": url,
        "GitBranch": branch,
    }
    tags = {
        k: v for k, v in tags.items() if v
    }
    return tags


if __name__ == '__main__':
    from pprint import pprint
    tags = git_get_tags(__file__)
    pprint(tags)
    from aws_sagemaker_remote.tags import make_tags
    tags = make_tags(tags)
    pprint(tags)




from sagemaker.session import Session as SagemakerSession
from boto3.session import Session as Boto3Session


def sagemaker_session(profile_name=None):
    if profile_name and len(profile_name) > 0:
        profile_name = profile_name
    else:
        profile_name = None
    boto_session = Boto3Session(profile_name=profile_name)
    sagemaker_session = SagemakerSession(boto_session=boto_session)
    return sagemaker_session
